//
//  FrontPageTableViewController.swift
//  CodeChix-Reddit
//
//  Created by Emily Van Haren on 3/8/15.
//  Copyright (c) 2015 Emily Van Haren. All rights reserved.
//

import UIKit

class FrontPageTableViewController: UITableViewController
{
    var activityIndicator: UIActivityIndicatorView!
    
    override init()
    {
        super.init()
        
        commonInit()
    }
    
    required init(coder aDecoder: NSCoder)
    {
        super.init(coder: aDecoder)
        
        commonInit()
    }
    
    override init(nibName nibNameOrNil: String?, bundle nibBundleOrNil: NSBundle?)
    {
        super.init(nibName: nibNameOrNil, bundle: nibBundleOrNil)
        
        commonInit()
    }
    
    override init(style: UITableViewStyle)
    {
        super.init(style: style)
        
        commonInit()
    }
    
    func commonInit()
    {
        NSNotificationCenter.defaultCenter().addObserver(self, selector: Selector("didGetRedditFrontPage"), name: "DidGetRedditFrontPage", object: nil)
    }
    
    override func viewDidLoad()
    {
        super.viewDidLoad()

        navigationItem.title = "Front Page"
        
        tableView.registerClass(PostTableViewCell.classForCoder(), forCellReuseIdentifier: "PostCell")
    }
    
    func didGetRedditFrontPage()
    {
        tableView.reloadData()
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
    }

    // MARK: - Table view data source

    override func numberOfSectionsInTableView(tableView: UITableView) -> Int
    {
        return 1
    }

    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int
    {
        if let allPosts = RedditServerInterface.interface.allPosts as NSArray?
        {
            return allPosts.count
        }
        else
        {
            return 0
        }
    }

    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell
    {
        let cell = tableView.dequeueReusableCellWithIdentifier("PostCell") as PostTableViewCell
        
        let thisPost = RedditServerInterface.interface.allPosts!.objectAtIndex(indexPath.row) as RedditPost
        cell.post = thisPost
        cell.initSubviews()
        cell.showSubviews()
        
        return cell
    }
    
    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat
    {
        let cell = PostTableViewCell()
        
        let thisPost = RedditServerInterface.interface.allPosts!.objectAtIndex(indexPath.row) as RedditPost
        cell.post = thisPost
        cell.initSubviews()
        
        return cell.heightForCellWithWidth(tableView.bounds.size.width)
    }
    
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath)
    {
        tableView.deselectRowAtIndexPath(indexPath, animated: false)
        
        let detailController = PostDetailViewController(post: RedditServerInterface.interface.allPosts!.objectAtIndex(indexPath.row) as RedditPost)
        navigationController?.pushViewController(detailController, animated: true)
    }
}
