//
//  PostTableViewCell.swift
//  CodeChix-Reddit
//
//  Created by Emily Van Haren on 3/9/15.
//  Copyright (c) 2015 Emily Van Haren. All rights reserved.
//

import UIKit

let PaddingHorizontal = 10.0 as CGFloat
let PaddingVertical = 15.0 as CGFloat
let PointsLabelFontSize = 11.0 as CGFloat
let PointsValueFontSize = 16.0 as CGFloat
let PointsWidth = 55.0 as CGFloat
let StarWidth = 20.0 as CGFloat
let SubredditFontSize = 12.0 as CGFloat
let ThumbnailWidth = 80.0 as CGFloat
let TitleFontSize = 16.0 as CGFloat

let FontName = "HelveticaNeue" as NSString

class PostTableViewCell: UITableViewCell
{
    var widthOfCell = 0.0 as CGFloat
    
    var post: RedditPost?
    
    var titleLabel: UILabel!
    var subredditLabel: UILabel!
    var pointsLabelLabel: UILabel!
    var pointsValueLabel: UILabel!
    
    var thumbnailImageView: UIImageView?
    
    var starButton: UIButton!
    var starView: StarView!
    
    override func layoutSubviews()
    {
        super.layoutSubviews()
        
        widthOfCell = contentView.bounds.size.width
        
        titleLabel.font = UIFont(name: FontName, size: TitleFontSize)
        titleLabel.frame = frameForTitleLabel()
        titleLabel.lineBreakMode = NSLineBreakMode.ByWordWrapping
        titleLabel.numberOfLines = 0
        titleLabel.textAlignment = NSTextAlignment.Left
        
        subredditLabel.font = UIFont(name: FontName, size: SubredditFontSize)
        subredditLabel.frame = frameForSubredditLabel()
        subredditLabel.textAlignment = NSTextAlignment.Left
        
        pointsLabelLabel.font = UIFont(name: FontName, size: PointsLabelFontSize)
        pointsLabelLabel.frame = frameForPointsLabelLabel()
        pointsLabelLabel.textAlignment = NSTextAlignment.Center
        
        pointsValueLabel.font = UIFont(name: FontName, size: PointsValueFontSize)
        pointsValueLabel.frame = frameForPointsValueLabel()
        pointsValueLabel.textAlignment = NSTextAlignment.Center
        
        thumbnailImageView?.frame = frameForThumbnailImage()
        
        starButton.frame = frameForStarButton()
        starView.frame = starButton.bounds
    }
    
    func heightForCellWithWidth(width: CGFloat) -> CGFloat
    {
        widthOfCell = width
        
        let titleHeight = frameForTitleLabel().size.height
        let subredditHeight = frameForSubredditLabel().size.height
        var totalHeight = (titleHeight + subredditHeight + (PaddingVertical * 3.0))
        
        if (post!.hasThumbnail())
        {
            let thumbnailHeight = (frameForThumbnailImage().size.height + (PaddingVertical * 2.0))
            if (thumbnailHeight > totalHeight)
            {
                totalHeight = thumbnailHeight
            }
        }
        
        return totalHeight
    }
    
    func initSubviews()
    {
        if (post != nil)
        {
            titleLabel = UILabel()
            titleLabel.text = post!.title
            
            subredditLabel = UILabel()
            subredditLabel.text = post!.subreddit
            
            pointsLabelLabel = UILabel()
            pointsLabelLabel.text = "points"
            
            pointsValueLabel = UILabel()
            pointsValueLabel.text = String(post!.score)
            
            if (post!.hasThumbnail())
            {
                thumbnailImageView = UIImageView(image: post!.thumbnailImage)
            }
            
            starView = StarView()
            starView.userInteractionEnabled = false
            starView.isSelected = post!.isStarred
            
            starButton = UIButton()
            starButton.addTarget(self, action: Selector("starButtonPressed"), forControlEvents: UIControlEvents.TouchUpInside)
        }
    }
    
    func showSubviews()
    {
        addSubview(titleLabel)
        addSubview(subredditLabel)
        addSubview(pointsLabelLabel)
        addSubview(pointsValueLabel)
        
        if (post!.hasThumbnail())
        {
            addSubview(thumbnailImageView!)
        }
        
        starButton.addSubview(starView)
        addSubview(starButton)
    }
    
    func frameForPointsLabelLabel() -> CGRect
    {
        let frameForPointsValue = frameForPointsValueLabel()
        
        let width = PointsWidth
        let height = PointsLabelFontSize
        let x = (contentView.bounds.size.width - (width + PaddingHorizontal))
        let y = (frameForPointsValue.origin.y + frameForPointsValue.size.height)
        
        return CGRectMake(x, y, width, height)
    }
    
    func frameForPointsValueLabel() -> CGRect
    {
        let width = PointsWidth
        let height = PointsValueFontSize
        let x = (contentView.bounds.size.width - (width + PaddingHorizontal))
        let y = PaddingVertical
        
        return CGRectMake(x, y, width, height)
    }
    
    func frameForStarButton() -> CGRect
    {
        let frameForPoints = frameForPointsLabelLabel()
        
        let width = StarWidth
        let height = width
        let x = (frameForPoints.origin.x + ((frameForPoints.size.width - width) / 2.0))
        let y = (frameForPoints.origin.y + frameForPoints.size.height + (PaddingVertical / 2.0));
        
        return CGRectMake(x, y, width, height)
    }
    
    func frameForSubredditLabel() -> CGRect
    {
        let titleFrame = frameForTitleLabel()
        
        let width = titleFrame.size.width
        let height = (SubredditFontSize * 1.25)
        let x = titleFrame.origin.x
        let y = (titleFrame.origin.y + titleFrame.size.height + PaddingVertical);
        
        return CGRectMake(x, y, width, height)
    }
    
    func frameForThumbnailImage() -> CGRect
    {
        if (post!.hasThumbnail())
        {
            let width = ThumbnailWidth
            
            let thumbnailSize = thumbnailImageView!.image!.size
            let scaleFactor = (width / thumbnailSize.width)
            let height = (thumbnailSize.height * scaleFactor)
            
            let x = PaddingHorizontal
            let y = PaddingVertical
            
            return CGRectMake(x, y, width, height)
        }
        else
        {
            return CGRectZero
        }
    }
    
    func frameForTitleLabel() -> CGRect
    {
        var widthForThumbnail = frameForThumbnailImage().size.width
        if (widthForThumbnail > 0)
        {
            widthForThumbnail += PaddingHorizontal
        }
        let widthForPoints = (frameForPointsValueLabel().size.width + PaddingHorizontal)
        
        let maximumWidth = widthOfCell
        let availableTextWidth = (maximumWidth - (widthForThumbnail + widthForPoints + (PaddingHorizontal * 2.0)))
        
        if let label = titleLabel as UILabel?
        {
            let labelAttributes = [NSFontAttributeName: UIFont(name: FontName, size: TitleFontSize)!]
            if let labelText = post?.title as NSString?
            {
                let labelRect = labelText.boundingRectWithSize(CGSizeMake(availableTextWidth, CGFloat.max), options: NSStringDrawingOptions.UsesLineFragmentOrigin, attributes: labelAttributes, context: nil)
                
                var x = (widthForThumbnail + PaddingHorizontal)
                if (widthForThumbnail == 0)
                {
                    x = PaddingHorizontal
                }
                let y = PaddingVertical
                
                return CGRectMake(x, y, labelRect.size.width, labelRect.size.height)
            }
            else
            {
                return CGRectZero
            }
        }
        else
        {
            return CGRectZero
        }
    }
    
    func starButtonPressed()
    {
        if post!.isStarred
        {
            post!.isStarred = false
            starView.isSelected = false
        }
        else
        {
            post!.isStarred = true
            starView.isSelected = true
        }
    }
    
    override func prepareForReuse()
    {
        pointsLabelLabel.removeFromSuperview()
        pointsValueLabel.removeFromSuperview()
        subredditLabel.removeFromSuperview()
        thumbnailImageView?.removeFromSuperview()
        titleLabel.removeFromSuperview()
        starButton.removeFromSuperview()
        starView.removeFromSuperview()
    }
}
