//
//  RedditServerInterface.swift
//  CodeChix-Reddit
//
//  Created by Emily Van Haren on 3/8/15.
//  Copyright (c) 2015 Emily Van Haren. All rights reserved.
//

import UIKit

class RedditServerInterface: NSObject, NSURLSessionDelegate, NSURLSessionTaskDelegate, NSURLSessionDataDelegate
{
    var allPosts: NSMutableArray?
    var dataTask: NSURLSessionDataTask!
    var incomingData: NSMutableData?
    var session: NSURLSession!
    var sessionConfig: NSURLSessionConfiguration!
    
    class var interface: RedditServerInterface
    {
        struct Static
        {
            static let interface: RedditServerInterface = RedditServerInterface()
        }
        return Static.interface
    }
    
    override init()
    {
        super.init()
        
        sessionConfig = NSURLSessionConfiguration.defaultSessionConfiguration()
        session = NSURLSession(configuration: sessionConfig, delegate: self, delegateQueue: NSOperationQueue.mainQueue())
    }
    
    func getFrontPage()
    {
        allPosts = NSMutableArray()
        incomingData = NSMutableData()
        
        let url = NSURL(string: "http://www.reddit.com/.json")
        
        var request = NSMutableURLRequest(URL: url!)
        request.HTTPMethod = "GET"
        
        dataTask = session.dataTaskWithRequest(request)
        dataTask.resume()
    }
    
    func URLSession(_session: NSURLSession, dataTask: NSURLSessionDataTask, didReceiveData data: NSData)
    {
        if (dataTask == self.dataTask)
        {
            incomingData?.appendData(data)
        }
    }
    
    func URLSession(session: NSURLSession, task: NSURLSessionTask, didCompleteWithError error: NSError?)
    {
        if (error == nil)
        {
            if let data = incomingData as NSMutableData?
            {
                if let json = NSJSONSerialization.JSONObjectWithData(data, options: NSJSONReadingOptions.MutableContainers, error: nil) as NSDictionary?
                {
                    if let jsonData = json.objectForKey("data") as NSDictionary?
                    {
                        if let jsonPosts = jsonData.objectForKey("children") as NSArray?
                        {
                            for jsonPost:NSDictionary in jsonPosts as Array
                            {
                                let post = RedditPost(json: jsonPost.objectForKey("data") as NSDictionary)
                                
                                allPosts?.addObject(post)
                            }
                            
                            if (allPosts?.count > 0)
                            {
                                NSNotificationCenter.defaultCenter().postNotificationName("DidGetRedditFrontPage", object: nil)
                            }
                        }
                        else
                        {
                            let alert = UIAlertView(title: "Server Error", message: "No posts to show!", delegate: self, cancelButtonTitle: "OK")
                            alert.show()
                        }
                    }
                    else
                    {
                        let alert = UIAlertView(title: "Server Error", message: "No data found!", delegate: self, cancelButtonTitle: "OK")
                        alert.show()
                    }
                }
                else
                {
                    let alert = UIAlertView(title: "Server Error", message: "No data found!", delegate: self, cancelButtonTitle: "OK")
                    alert.show()
                }
            }
            else
            {
                let alert = UIAlertView(title: "Server Error", message: "No data found!", delegate: self, cancelButtonTitle: "OK")
                alert.show()
            }
        }
        else
        {
            let alert = UIAlertView(title: "Error", message: "No internet connection!", delegate: self, cancelButtonTitle: "OK")
            alert.show()
        }
    }
}
