//
//  StarView.swift
//  CodeChix-Reddit
//
//  Created by Emily Van Haren on 3/9/15.
//  Copyright (c) 2015 Emily Van Haren. All rights reserved.
//

import UIKit

class StarView: UIView
{
    var _isSelected: Bool = false
    var isSelected: Bool
    {
        get
        {
            return _isSelected
        }
        set
        {
            _isSelected = newValue
            setNeedsDisplay()
        }
    }
    
    override init()
    {
        super.init()
        
        commonInit()
    }
    
    required init(coder aDecoder: NSCoder)
    {
        super.init(coder: aDecoder)
        
        commonInit()
    }
    
    override init(frame: CGRect)
    {
        super.init(frame: frame)
        
        commonInit()
    }
    
    func commonInit()
    {
        backgroundColor = UIColor.clearColor()
    }
    
    override func drawRect(rect: CGRect)
    {
        let context = UIGraphicsGetCurrentContext()
        
        if isSelected
        {
            UIColor.orangeColor().setFill()
        }
        else
        {
            UIColor(white: 0.0, alpha: 0.25).setFill()
        }
        
        let centerX = (bounds.size.width * 0.5) as CGFloat
        let centerY = (bounds.size.height * 0.5) as CGFloat
        
        let radius = centerX
        let flip = -1.0 as CGFloat
        let theta = (2.0 * (CGFloat(M_PI) * (2.0 / 5.0))) as CGFloat // 144 degrees
        
        let startY = ((radius * flip) + centerY)
        CGContextMoveToPoint(context, centerX, startY)
        
        for i in 1...5
        {
            let x = ((radius * sin(CGFloat(i) * theta)) + centerX)
            let y = (((radius * cos(CGFloat(i) * theta)) * flip) + centerY)
            
            CGContextAddLineToPoint(context, x, y)
        }
        
        CGContextFillPath(context)
    }
}
